<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */
  
namespace FiloBlu\Esb\Setup;
 
use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Ddl\Table;
 
class UpgradeSchema implements UpgradeSchemaInterface
{
    protected $_statusFactory;
    public function __construct(
        \Magento\Sales\Model\Order\StatusFactory $statusFactory
    ){
        $this->_statusFactory = $statusFactory;
    }
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();
        
        if (version_compare($context->getVersion(), '1.0.1') < 0) {

            // Get module table
            $tableName = $installer->getTable('esb_schedule');
            $connection = $installer->getConnection();
            if ($setup->getConnection()->isTableExists($tableName) == true) {
                // Declare data
                $connection->addColumn( 
                    $tableName,
                    'object_repository_interface', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'nullable' => true,
                        'comment' => 'Object Repository Interface',
                        'after' => 'object_class'
                    ]
                );
                $connection->modifyColumn(
                    $tableName,
                    'object_class',                    
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'comment' => 'Object Class',
                        'nullable' => true
                    ]
                );
            }
        }
        if (version_compare($context->getVersion(), '1.0.2') < 0) {
            $confirmedStatus = $this->_statusFactory->create();
            $confirmedStatus->setData('status', 'confirmed');
            $confirmedStatus->setData('label', 'Confirmed');
            $confirmedStatus->save();            
            $confirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);

            $orderExportedStatus = $this->_statusFactory->create();
            $orderExportedStatus->setData('status', 'order_exported');
            $orderExportedStatus->setData('label', 'Order Exported');
            $orderExportedStatus->save();
            $orderExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, false);

            $paymentConfirmedStatus = $this->_statusFactory->create();
            $paymentConfirmedStatus->setData('status', 'payment_confirmed');
            $paymentConfirmedStatus->setData('label', 'Payment Confirmed');
            $paymentConfirmedStatus->save();
            $paymentConfirmedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);   

            $paymentExportedStatus = $this->_statusFactory->create();
            $paymentExportedStatus->setData('status', 'payment_exported');
            $paymentExportedStatus->setData('label', 'Payment Exported');
            $paymentExportedStatus->save();
            $paymentExportedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, false);                     

            $invoicedStatus = $this->_statusFactory->create();
            $invoicedStatus->setData('status', 'invoiced');
            $invoicedStatus->setData('label', 'Invoiced');
            $invoicedStatus->save();            
            $invoicedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);

            $shippedStatus = $this->_statusFactory->create();
            $shippedStatus->setData('status', 'shipped');
            $shippedStatus->setData('label', 'Shipped');
            $shippedStatus->save();            
            $shippedStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);              
            
        }
        if (version_compare($context->getVersion(), '1.0.5') < 0) {
            $tableName = $installer->getTable('esb_schedule');
            $connection = $installer->getConnection();
            if ($setup->getConnection()->isTableExists($tableName) == true) {
                // Declare data
                $connection->addColumn( 
                    $tableName,
                    'object_after_method', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'nullable' => true,
                        'comment' => 'Method to be executed after export',
                        'after' => 'event'
                    ]
                );
                $connection->addColumn( 
                    $tableName,
                    'object_after_method_argument', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'default' => NULL,
                        'nullable' => true,
                        'comment' => 'Method to be executed after export (argument)',
                        'after' => 'object_after_method'
                    ]
                );                
            }
        }
        if (version_compare($context->getVersion(), '1.0.19') < 0) {
            $tableName = $installer->getTable('esb_schedule');
            $connection = $installer->getConnection();
            if ($setup->getConnection()->isTableExists($tableName) == true) {
                $connection->addColumn( 
                    $tableName,
                    'priority', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                        'unsigned'  => true,
                        'default' => 0,
                        'nullable' => false,
                        'comment' => 'Priority',
                        'after' => 'response'
                    ]
                );                
            }
        }    
        if (version_compare($context->getVersion(), '1.0.21') < 0) {
            $newStatus = $this->_statusFactory->create();
            $newStatus->setData('status', 'sent_to_logistics');
            $newStatus->setData('label', 'Sent to Logistics');
            $newStatus->save();            
            $newStatus->assignState(\Magento\Sales\Model\Order::STATE_PROCESSING, false, true);     
        }            
        if (version_compare($context->getVersion(), '1.1.8') < 0) {
            $tableName = $installer->getTable('esb_schedule');
            $connection = $installer->getConnection();
            if ($setup->getConnection()->isTableExists($tableName) == true) {
                $connection->addColumn( 
                    $tableName,
                    'sendmessage_class', 
                    [
                        'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                        'length' => 255,
                        'default' => '\FiloBlu\Esb\Cron\SendMessage',
                        'nullable' => true,
                        'comment' => 'SendMessage class',
                        'after' => 'event'
                    ]
                );                
            }
        }
        $installer->endSetup();
    }
}